@@ -1,10 +1,13 @@
 
 
+import datetime
+import inspect
 import json
 import logging
 from socketserver import ThreadingMixIn
 from xmlrpc.server import SimpleXMLRPCServer
 from xmlrpc.server import SimpleXMLRPCRequestHandler
+from xmlrpc.server import resolve_dotted_attribute
 
 from fastutils import logutils
 from daemon_application import DaemonApplication
@@ -34,6 +37,8 @@ class SimpleXmlRpcRequestHandler(SimpleXMLRPCRequestHandler):
     """
     supported server settings:
 
+    server-tokens: false
+
     apikey-auth-header: apikey
     apikeys:
       app1:
@@ -66,7 +71,7 @@ class SimpleXmlRpcRequestHandler(SimpleXMLRPCRequestHandler):
         return self.server.config.get("apikey-auth-header", "apikey")
 
     def get_apikeys(self):
-        return self.server.config.get("get_apikeys", {})
+        return self.server.config.get("apikeys", {})
 
     def send_response(self, code, message=None):
         self.log_request(code)
@@ -79,7 +84,10 @@ class SimpleXmlRpcRequestHandler(SimpleXMLRPCRequestHandler):
             self.send_header("Connection", "Keep-Alive")
             keepalive_timeout = self.get_keepalive_timeout()
             keepalive_max = self.get_keepalive_max()
-            self.send_header("Keep-Alive", f"timeout={keepalive_timeout}, max={keepalive_max}")
+            self.send_header("Keep-Alive", "timeout={keepalive_timeout}, max={keepalive_max}".format(
+                keepalive_timeout=keepalive_timeout,
+                keepalive_max=keepalive_max,
+            ))
         self.send_header("Date", self.date_time_string())
 
     def do_POST(self):
@@ -115,6 +123,79 @@ class SimpleThreadedXmlRpcServer(ThreadingMixIn, SimpleXMLRPCServer):
     def set_config(self, config):
         self.config = config
 
+    def system_methodSignature(self, method_name):
+        # try to get methodSignature from doc string
+        help_text = self.system_methodHelp(method_name) or ""
+        returns = None
+        returns_flag = False
+        args = None
+        args_flag = False
+        for line in help_text.splitlines():
+            line = line.strip()
+            if line.startswith("@methodSignature"):
+                return json.loads(line.split(":")[1].strip())
+            if line.startswith("@signature"):
+                return json.loads(line.split(":")[1].strip())
+            if line.startswith("@returns"):
+                returns = line.split(":")[1].strip()
+                returns_flag = True
+            if line.startswith("@args"):
+                args = json.loads(line.split(":")[1].strip())
+                args_flag = True
+        if returns_flag and args_flag:
+            return [returns] + list(args)
+        
+        # try to get methodSignature from _meta attributes
+        method = self._get_method(method_name)
+        if method is None:
+            return "signatures not supported"
+
+        if hasattr(method, "_methodSignature"):
+            return getattr(method, "_methodSignature")
+        if hasattr(method, "_signature"):
+            return getattr(method, "_signature")
+        if hasattr(method, "_returns") and hasattr(method, "_args"):
+            return [getattr(method, "_returns")] + list(getattr(method, "_args"))
+
+        # None.__name__ will cuase error
+        def get_class_name(klass):
+            if klass is None:
+                return "nil"
+            return klass.__name__
+        # try to get methodSignature from typing
+        result = []
+        result_flag = True
+        signature = inspect.signature(method)
+        if signature.return_annotation == inspect._empty:
+            result_flag = False
+        else:
+            result.append(get_class_name(signature.return_annotation))
+            for _, klass in signature.parameters.items():
+                if not klass.annotation:
+                    result_flag = False
+                    break
+                result.append(get_class_name(klass.annotation))
+        if result_flag:
+            return result
+        else:
+            return "signatures not supported"
+
+    def _get_method(self, method_name):
+        method = None
+        if method_name in self.funcs:
+            method = self.funcs[method_name]
+        elif self.instance is not None:
+            if not hasattr(self.instance, '_dispatch'):
+                try:
+                    method = resolve_dotted_attribute(
+                                self.instance,
+                                method_name,
+                                self.allow_dotted_names
+                                )
+                except AttributeError:
+                    pass
+        return method
+
 class SimpleXmlRpcServer(DaemonApplication):
 
     def get_default_listen_port(self):
@@ -126,22 +207,26 @@ class SimpleXmlRpcServer(DaemonApplication):
     def get_request_handler_class(self):
         return SimpleXmlRpcRequestHandler
 
-    def main(self):
-        logutils.setup(**self.config)
-        self.server_listen = tuple(self.config.get("server", {}).get("listen", ("0.0.0.0", self.get_default_listen_port())))
-        logger.warn("Starting xmlrpc server on {server_listen}...".format(server_listen=self.server_listen))
-        self.server = SimpleThreadedXmlRpcServer(
-            self.server_listen,
+    def make_server(self):
+        server_listen = tuple(self.config.get("server", {}).get("listen", ("0.0.0.0", self.get_default_listen_port())))
+        server = SimpleThreadedXmlRpcServer(
+            server_listen,
             requestHandler=self.get_request_handler_class(),
             allow_none=True,
             encoding="utf-8",
             )
-        self.server.set_config(self.config)
+        server.set_config(self.config)
+        return server, server_listen
+
+    def main(self):
+        logutils.setup(**self.config)
+        self.server, self.server_listen = self.make_server()
         self.register_services()
         try:
+            logger.info("Starting xmlrpc server on {server_listen}...".format(server_listen=self.server_listen))
             self.server.serve_forever()
         except KeyboardInterrupt:
-            logger.warn("Got KeyboardInterrupt signal, stopping the service...")
+            logger.info("Got KeyboardInterrupt signal, stopping the service...")
 
     def register_services(self):
         disable_debug_service_flag = self.get_disable_debug_service_flag()
